(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

(* file version: 9/20/92 *)

(*:Version: Mathematica 2.0 *)
(*:Name: 	BayesianEmetrics *)
(*:Title: 	Tools for Bayesian Econometrics *)
(*:Author: 	Eduardo Ley and Mark F.J. Steel, July 1992 *)
(*:Legal: 	Copyright (c) 1992, Eduardo Ley and Mark F.J. Steel *)
(*:Summary: *)
(*:Keywords: Bayesian, Regression *)
(*:Requirements: No special system requirements. *)
(*:Warning: None. *)
(*:Sources: 
	A. Zellner (1971): "An Introduction to Bayesian Inference in Econometrics,"
		NY: Wiley.
	G.G. Judge, R.C. Hill, W.E. Griffiths, H. Lutkepohl, T.-C. Lee (1982):
		"Introduction to the Theory and Practice of Econometrics," NY: Wiley.  
*)

BeginPackage["BayesianEmetrics`",
			 "Statistics`ContinuousDistributions`",
	  		 "Statistics`NormalDistribution`", 
  			 "Statistics`InverseStatisticalFunctions`", 
  			 "Statistics`Common`DistributionsCommon`", 
  			 "Statistics`DescriptiveStatistics`",
  			 "Statistics`DataManipulation`",
  			 "Graphics`Graphics`",
  			 "Graphics`Graphics3D`"  			 
  			 ]

IGammaDistribution::usage =
"IGammaDistribution[alpha,beta] represents the Inverted Gamma (Type-2)
Distribution as in 
	H. Raiffa and R. Schlaifer (1961):
	Applied Statistical Decision Theory;
	Boston: Harvard Business School; or
	Arnold Zellner (1971):
	Bayesian Inference in Econometrics,
	pp. 369-373; New York: Wiley."

MultivariateTDistribution::usage =
"MultivariateTDistribution[m, H, v] represents a multivariate t
distribution with mean vector m, and covariance matrix (v/(v-2))Inverse[H].
	Arnold Zellner (1971):
	Bayesian Inference in Econometrics,
	pp. 383-389; New York: Wiley."

UnivariateTDistribution::usage =
"UnivariateTDistribution[m, h, v] represents a univariate t
distribution with mean vector m, and variance (v/(v-2))(1/h).
	Arnold Zellner (1971):
	Bayesian Inference in Econometrics,
	pp. 366-367; New York: Wiley."

MultivariateNormalDistribution::usage =
"MultivariateNormalDistribution[mu, sigma] represents a multivariate
Normal distribution with mean mu and covariance matrix sigma."

Likelihood::usage =
"Likelihood[density,data] calculates the
 likelihood function of the data using the specified
 density---ie, the PDF of the density is evaluated
 at each data point and the product computed."

LogLikelihood::usage =
"LogLikelihood[density,data] calculates the
 log-likelihood function of the data using the 
 specified density---ie, the log[PDF] of the density 
 is evaluated at each data point and the sum computed."

Marginal::usage =
"Marginal[ list, MultivariateDistribution[ parameters ]], where list is a 
list of indices and MultivariateDistribution is either  
MultivariateTDistribution or MultivariateNormal, gives the marginal 
distribution of the variables whose indices are listed in list."

BayesRegression::usage =
"BayesRegression[y, x, priormeanbeta, A, priormeansigma, n, xf] computes the 
posterior distribution for beta, sigma (and future values of y if xf is 
provided).  If prior means for beta and sigma are provided along with the 
precision matrix A, and n, then it assumed that the priors are natural 
conjugates.  Otherwise, diffuse priors are assumed."

postdistbeta::usage =
"It is returned by BayesRegression and contains the
posterior distribution of the regression vector, beta."

postdistsigma::usage =
"It is returned by BayesRegression and contains the
posterior distribution of the error standard deviation, sigma."

preddisty::usage=
"It is returned by BayesRegression when xf is specified 
and contains the posterior distribution of the predicted values of y."

HHistogram::usage=
"Histogram[xdata, <numberofbins>] returns a HORIZONTAL Histogram of xdata.
*** WARNING *** there's a bug in BinCounts and sometimes the number
of bins is not what Histogram expects, try multiplying your data by
some constant like 100 or 1000."

VHistogram::usage=
"Histogram[xdata, <numberofbins>] returns a VERTICAL Histogram of xdata.
Only two decimals will be shown in the x axis.  Multiply the data by some
constant or use VHistogram if that's not enough precision.
*** WARNING *** there's a bug in BinCounts and sometimes the number
of bins is not what Histogram expects, try multiplying your data by
some constant like 100 or 1000."

SmoothedVHistogram::usage=
"Histogram[xdata, <numberofbins>] returns a SMOOTHED VERTICAL Histogram of xdata.
*** WARNING *** there's a bug in BinCounts and sometimes the number
of bins is not what Histogram expects, try multiplying your data by
some constant like 100 or 1000."

Histogram3D::usage=
"Histogram[xydata, <numberofbins>, opts] uses BarChart3D and ListOfBinCounts
to construct a 3D Histogram. 
*** WARNING *** there's a bug in BinCounts and sometimes the number
of bins is not what Histogram3D expects, try multiplying your data by
some constant like 100 or 1000."

ListOfBinCoordinates::usage=
"It returns the x and y coordinates at the bins' center.  It is used in
conjunction with Histogram3D since it displays only the bin number and not
its coordinates."

ListOfBinCounts::usage=
"ListOfBinCounts[xy_List, n_Integer] is an auxiliary function that calls 
BinCounts and returns a list of n bin counts of xy."

(********************************************************************* 
	Begin Private Context
*********************************************************************)

Begin["`Private`"]

(********************************************************************* 
	Inverted Gamma Distribution
	Commonly used in Bayesian Statistics,
	see H. Raiffa and R. Schlaifer (1961):
	"Applied Statistical Decision Theory;" 
	Boston: Harvard Business School; or
	Arnold Zellner (1971), pp. 369-373.
***********************************************************************)

IGammaDistribution/:
	Domain[IGammaDistribution[___]] := {0,Infinity}

IGammaDistribution/:
	PDF[IGammaDistribution[v_, s_], x_] := 
		With[{result = (2*(1/2)^(v/2)*x^(-1-v)*(s^2*v)^(v/2))/
  			(E^((s^2*v)/(2*x^2))*Gamma[v/2])},
		If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	CDF[IGammaDistribution[v_, s_], x_] :=
		With[{result = (Gamma[v/2] - 
		Gamma[v/2, 0, (v*s^2)/(2*x^2)])/Gamma[v/2]},
		If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	Mean[IGammaDistribution[v_, s_]] :=
		With[{result = Sqrt[v/2]Gamma[(v-1)/2]s/Gamma[v/2]},
		If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	Variance[IGammaDistribution[v_, s_]] :=
		With[{result = ((v*s^2)/(v-2))-
		(Mean[IGammaDistribution[v,s]])^2},
		If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	StandardDeviation[IGammaDistribution[v_, s_]] :=
		With[{result = Sqrt[Variance[IGammaDistribution[v,s]]]},
		If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	Skewness[IGammaDistribution[n_,s_]] :=
		With[{result = (n^(3/2)*s^3*Gamma[(-1 + n)/2]^3)/
		(2^(1/2)*Gamma[n/2]^3) + (n^(3/2)*s^3*Gamma[(-3 + n)/2])/
   		(2^(3/2)*Gamma[n/2]) - (3*2^(3/2)*n^(3/2)*s*Gamma[(-1 + n)/2])/
  		((-2 + n)*Gamma[n/2])},
		If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	Kurtosis[IGammaDistribution[v_, s_]] :=
	With[{result =(s^4*v^2)/(8 - 6*v + v^2) - 
	(3*n^2*s^4*Gamma[(-1 + n)/2]^4)/(4*Gamma[n/2]^4) - 
   	(n^2*s^4*Gamma[(-3 + n)/2]*Gamma[(-1 + n)/2])/
   	Gamma[n/2]^2 + (12*n^2*s^2*Gamma[(-1 + n)/2]^2)/((-2 + n)*Gamma[n/2]^2)},
   	If[NumberQ[N[result]],N[result],result]]

IGammaDistribution/:
	KurtosisExcess[IGammaDistribution[v_, s_]] :=
		NotImplemented/; False;

IGammaDistribution/:
	CharacteristicFunction[IGammaDistribution[v_, s_]] :=
		NotImplemented/; False;

IGammaDistribution/:
	Quantile[IGammaDistribution[v_, s_], q_] :=
		x/.FindRoot[q-CDF[IGammaDistribution[v,s],x],
		{x,1,0,v*s*1000}]  

IGammaDistribution/:
	Random[IGammaDistribution[v_, s_]] :=
	With[{result = Quantile[IGammaDistribution[v,s],Random[]]},
		If[NumberQ[N[result]],N[result],result]]

(********************************************************************* 
	Univariate Student t distribution. See
	Arnold Zellner (1971), pp. 366-367.
***********************************************************************)

UnivariateTDistribution/:
	Domain[UnivariateTDistribution[___]] := {-Infinity, Infinity}

UnivariateTDistribution/:
	PDF[UnivariateTDistribution[m_, h_, v_], x_] := 
		With[{result = (Gamma[(v+1)/2]*Sqrt[h/v])/
		(Sqrt[Pi]*Gamma[v/2]*(1+(h/v)*(x-m)^2)^((v+1)/2))},
		If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	CDF[UnivariateTDistribution[m_, h_, v_], x_] :=
		With[{result = CDF[StudentTDistribution[v], Sqrt[h]*(x-m)]},
		If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	Mean[UnivariateTDistribution[m_, h_, v_]] :=
		With[{result = m},
		If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	Variance[UnivariateTDistribution[m_, h_, v_]] :=
		With[{result = (v/(v-2))*(1/h)},
		If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	StandardDeviation[UnivariateTDistribution[m_, h_, v_]] :=
		With[{result = Sqrt[(v/(v-2))*(1/h)]},
		If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	Skewness[UnivariateTDistribution[ n_, s_]] := 0

UnivariateTDistribution/:
	Kurtosis[UnivariateTDistribution[m_, h_, v_]] :=
	With[{result =(3/((v-2)(v-4)))*(v/h)^2},
   	If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	KurtosisExcess[UnivariateTDistribution[m_, h_, v_]] :=
	With[{result =6/(v-4)},
   	If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	CharacteristicFunction[UnivariateTDistribution[m_, h_, v_]] :=
		NotImplemented/; False;

UnivariateTDistribution/:
	Quantile[UnivariateTDistribution[m_, h_, v_], q_] :=
	With[{result = (Quantile[StudentTDistribution[v], q]/Sqrt[h])+ m},
		If[NumberQ[N[result]],N[result],result]]

UnivariateTDistribution/:
	Random[UnivariateTDistribution[m_, h_, v_]] :=
	Quantile[UnivariateTDistribution[m, h, v], Random[]]

(**************************************************************** 
	Multivariate t Density 
 	As defined in A. Zellner (1971), pp. 383-389.
 ****************************************************************)

MultivariateTDistribution/:
	Domain[MultivariateTDistribution[___]] := {-Infinity,Infinity}

MultivariateTDistribution/:
	PDF[MultivariateTDistribution[mean_, H_, v_], x_] := 
		Module[{k, qform}, 
			k = Max[Length[mean],TensorRank[H]];
			qform = Transpose[x-mean].H.(x-mean);
			With[{result = (Gamma[(v+k)/2]*Sqrt[Det[H]/v^k])/
			(Sqrt[Pi^k]*Gamma[v/2]*Sqrt[(1+qform/v)^(v+k)])},
			If[NumberQ[N[result]],N[result],result]]
			]

MultivariateTDistribution/:
	Mean[MultivariateTDistribution[mean_, H_, v_]] :=
	With[{result = mean},
		If[NumberQ[N[result]],N[result],result]]

MultivariateTDistribution/:
	Variance[MultivariateTDistribution[mean_, H_, v_]] :=
	With[{result = (v/(v-2))*Inverse[H]},
		If[NumberQ[N[result]],N[result],result]]

MultivariateTDistribution/:
Marginal[ijk_List, MultivariateTDistribution[mean_, H_, v_]]:=
	Module[{inH, Hijk,lenz},
		inH = Inverse[H];
		Hijk = Inverse[inH[[ijk,ijk]]];
		Length[ijk];
		If[Length[ijk]==1, 
			UnivariateTDistribution[Flatten[mean[[ijk]]][[1]], 
			Flatten[1/inH[[ijk,ijk]]][[1]], v],
			MultivariateTDistribution[mean[[ijk]], Hijk, v]
		]
	]

(********************************************************************
		Multivariate Normal 
********************************************************************)

MultivariateNormalDistribution/:
	Domain[MultivariateNormalDistribution[___]] := {-Infinity,Infinity}

MultivariateNormalDistribution/:
	PDF[MultivariateNormalDistribution[mu_,A_], x_] := 
		Module[{k}, 
		k = Max[Length[mean],TensorRank[A]];
		With[{result = 1/Sqrt[(2Pi)^k Det[A]]*
			Exp[0.5*Transpose[x-mu].Inverse[A].(x-mu)]},
			If[NumberQ[N[result]],N[result],result]]
		]

MultivariateNormalDistribution/:
	Mean[MultivariateNormalDistribution[mu_, A_]] :=
	With[{result = mu},
		If[NumberQ[N[result]],N[result],result]]

MultivariateNormalDistribution/:
	Variance[MultivariateNormalDistribution[mu_, A_]] :=
	With[{result = A},
		If[NumberQ[N[result]],N[result],result]]

MultivariateNormalDistribution/:
Marginal[ijk_List, MultivariateNormalDistribution[mu_, A_]]:=
	If[Length[ijk]==1,
	NormalDistribution[Flatten[mu[[ijk]]][[1]], Flatten[A[[ijk,ijk]]][[1]]],
	MultivariateNormalDistribution[mu[[ijk]], A[[ijk,ijk]]]
	]

(********************************************************************
		Likelihood Function
********************************************************************)

Likelihood[density_, data_] :=
 	Module[{i}, Product[PDF[density,data[[i]]],{i,1,Length[data]}]]

LogLikelihood[density_,data_] := Log[LikelihoodFunction[density, data]]

(********************************************************************
		Bayesian Regression 
********************************************************************)

BayesRegression[y_, x_] := 
	Module[{betahat, nu, ehat, sigma2hat, ivar},
		betahat  = N[Inverse[Transpose[x].x].Transpose[x].y];
		nu = Length[y]-Length[betahat];
		ehat = y - x.betahat;
		sigma2hat  = N[(Transpose[ehat].ehat)/nu];
		ivar  = N[Transpose[x].x/sigma2hat];
		postdistbeta = MultivariateTDistribution[betahat, ivar, nu];
		postdistsigma = IGammaDistribution[nu,Sqrt[sigma2hat]];
	]

BayesRegression[y_, x_, xf_] := 
	Module[{mean, ivar, ivaryf, betahat, sigma2hat, nu},
		betahat  = N[Inverse[Transpose[x].x].Transpose[x].y];
		mean  = N[xf.betahat];
		nu = Length[y]-Length[betahat];
		ehat = y - x.betahat;
		sigma2hat  = N[(Transpose[ehat].ehat)/nu];
		ivar  = N[Transpose[x].x/sigma2hat];
		ivaryf  = N[(1/sigma2hat)*
				Inverse[IdentityMatrix[Part[Dimensions[xf],1]] +
				xf.Inverse[Transpose[x].x].Transpose[xf]]];
				postdistbeta = MultivariateTDistribution[betahat, ivar, nu];
				postdistsigma = IGammaDistribution[nu,Sqrt[sigma2hat]];
				preddisty = MultivariateTDistribution[mean, ivaryf, nu];
		]

BayesRegression[y_, x_, priorbeta_, A_, priorsigma_, priorn_]:= 
	Module[{postn, postbeta, postsigma, ivar},
		postn = priorn + Length[y];
		postbeta  = N[Inverse[A+Transpose[x].x].(A.priorbeta+Transpose[x].y)];
	    postsigma = N[Sqrt[(priorn*priorsigma^2 + Transpose[y].y - 
	            Transpose[postbeta].(A+Transpose[x].x).postbeta +
	            Transpose[priorbeta].A.priorbeta)/postn]];
	    ivar  = N[(A+Transpose[x].x)*postsigma^(-2)];	
		postdistbeta = MultivariateTDistribution[postbeta, ivar, postn];
		postdistsigma = IGammaDistribution[postn, postsigma];
	]

BayesRegression[y_, x_, MultivariateTDistribution[m_, H_, n_],
IGammaDistribution[n_, s_]] := BayesRegression[y, x, m, H*s^2, s, n];

BayesRegression[y_, x_, priorbeta_, A_, priorsigma_,  priorn_, xf_]:= 
	Module[{mean, ivar, ivaryf, postn, postbeta, postsigma},
	    postn = priorn + Length[y];
		postbeta  = N[Inverse[A+Transpose[x].x].(A.priorbeta+Transpose[x].y)];
	    postsigma = N[Sqrt[(priorn*priorsigma^2 + Transpose[y].y - 
	            Transpose[postbeta].(A+Transpose[x].x).postbeta +
	            Transpose[priorbeta].A.priorbeta)/postn]];
		mean = N[xf.postbeta];
		ivar  = N[(A+Transpose[x].x)*postsigma^(-2)];
		ivaryf = N[postsigma^(-2)*
			Inverse[IdentityMatrix[Part[Dimensions[xf],1]] +
			xf.Inverse[A+Transpose[x].x].Transpose[xf]]];
		postdistbeta  = MultivariateTDistribution[postbeta, ivar, postn];
		postdistsigma = IGammaDistribution[postn, postsigma];			
		preddisty     = MultivariateTDistribution[mean, ivaryf, postn];
	]

BayesRegression[y_, x_, MultivariateTDistribution[m_, H_, n_],
IGammaDistribution[n_, s_], xf] := BayesRegression[y, x, m, H*s^2, s, n, xf]

(***********************************************************************
      	Histograms
***********************************************************************)

HHistogram[x_List, n_Integer:10, opts___Rule] :=
Module[{dx, maxx, minx, area},
	maxx = Max[x];
	minx = Min[x];
	dx = (maxx - minx)/n;
	area = dx * Length[x];
	BarChart[Transpose[{BinCounts[x, {minx, maxx, dx}]/area,
	Table[N[i, 8], {i, maxx - dx/2, minx + dx/2, -dx}]}],
	BarOrientation->Horizontal, opts]]

HHistogram[x_List, opts___Rule] := HHistogram[x, 10, opts]

VHistogram[x_List, n_Integer:10, opts___Rule] :=
Module[{dx, maxx, minx, area},
	maxx = Max[x];
	minx = Min[x];
	dx = (maxx - minx)/n;
	area = dx * Length[x];
	BarChart[Transpose[{BinCounts[x, {minx, maxx, dx}]/area,
	Table[N[i, 2], {i, minx + dx/2, maxx - dx/2, dx}]}],
	opts]]	

VHistogram[x_List, opts___Rule] := VHistogram[x, 10, opts]

SmoothedVHistogram[x_List, n_Integer:10, opts___Rule] :=
Module[{dx, maxx, minx, bc, xc, z, hist, area},
	maxx = Max[x];
	minx = Min[x];
	dx = (maxx-minx)/n;
	area = dx * Length[x];
	(*we create two extra bins at the extremes*)
	(*we rescale so that it integrates to 1*)
	bc = BinCounts[x, {minx - 2*0.9*dx, maxx + 2*0.9*dx, dx}]/area;
	xc = Range[minx - (3/2)*dx, maxx + (3/2)*dx, dx];
	z = Transpose[ColumnJoin[{xc}, {bc}]];
	hist = Interpolation[z];
	Plot[hist[y], {y, minx, maxx}, opts]	
	]

SmoothedVHistogram[x_List, opts___Rule] := SmoothedVHistogram[x, 10, opts]

ListOfBinCounts[xy_List, n_Integer:10] :=
Module[{x, y, dx, dy, maxx, minx, maxy, miny},
	x = Transpose[xy][[1]];
	y = Transpose[xy][[2]];
	maxx = Max[x];
	minx = Min[x];
	maxy = Max[y];
	miny = Min[y];
	dx = (maxx-minx)/n;
	dy = (maxy-miny)/n;
	BinCounts[xy, {minx, maxx, dx}, {miny, maxy, dy}]
	]

Histogram3D[xy_List, n_Integer:10, opts___Rule] :=  
BarChart3D[ListOfBinCounts[xy, n], opts]

	Histogram3D[xy_List, opts___] := Histogram3D[xy, 10, opts]

ListOfBinCoordinates[xy_List, n_Integer:10] :=
Module[{x, y, dx, xx, yy, maxx, minx, maxy, miny},
	x = Transpose[xy][[1]];
	y = Transpose[xy][[2]];
	maxx = Max[x];
	minx = Min[x];
	maxy = Max[y];
	miny = Min[y];
	dx = (maxx-minx)/n;
	dy = (maxy-miny)/n;
	xx = Range[minx, maxx, dx];
	yy = Range[miny, maxy, dy];
	TableForm[Table[{N[xx[[i]], 4], N[yy[[i]], 4]}, {i, 1, n}],
	TableHeadings->{Automatic, {"x coord.", "y coord."}}] 
	]

(********************************************************************
		End of Private Context 
********************************************************************)

End[]

(********************************************************************
		HouseKeeping
********************************************************************)

SetAttributes[IGammaDistribution, 				ReadProtected];
SetAttributes[UnivariateTDistribution, 			ReadProtected];
SetAttributes[MultivariateTDistribution, 		ReadProtected];
SetAttributes[MultivariateNormalDistribution, 	ReadProtected];
SetAttributes[BayesRegression, 					ReadProtected];
SetAttributes[Marginal, 						ReadProtected];
SetAttributes[Likelihood, 						ReadProtected];
SetAttributes[LogLikelihood, 					ReadProtected];
SetAttributes[VHistogram, 						ReadProtected];
SetAttributes[HHistogram, 						ReadProtected];
SetAttributes[SmoothedVHistogram, 				ReadProtected];
SetAttributes[Histogram3D, 						ReadProtected];
SetAttributes[ListOfBinCoordinates, 			ReadProtected];
SetAttributes[ListOfBinCounts, 					ReadProtected];

Protect[IGammaDistribution, 		UnivariateTDistribution,
		MultivariateTDistribution, 	MultivariateNormalDistribution,
		BayesRegression,			Marginal,
		Likelihood,					LogLikelihood,
		VHistogram,					SmoothedVHistogram,
		HHistogram,					Histogram3D,	
		ListOfBinCoordinates,		ListOfBinCounts
		];

(********************************************************************
		End of Package 
********************************************************************)

EndPackage[]