(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

(*:Version: Mathematica 2.0 *)
(*:Name: EconometricsAndFinance`JohnsonDistributions *)
(*:Title: Johnson System of Continuous Distributions From
          Four Moments  *)
(*:Author:
  Bob Korsan (Decisions, Decisions), July, 1992.
*)
(*:Reference: Usage messages only. *)
(*:Summary:
  Properties and functionals of continuous probability distributions
  used in statistical computations. These three distributions are
  sufficient to match the first four moments of any distribution.
  
  This package contains programs from the article, 'Decision 
  Analytica' in the book "Econometrics and Finance in Mathematica" edited
  by Dr. Hal Varian, Springer Verlag, The Electronic Library Of Science,
  Santa Clara, CA.  ISBN 0-387-97882-8.

Any comments, bug reports, or requests should be forwarded to:
	Robert J. Korsan
	Decisions, Decisions!
	2136 Lyon Avenue
	Belmont, CA 94002-1639
	bobk@decide.com
	1 (415) 595-1206
*)
(* :Context: DecisionAnalytica`JohnsonDistributions` 
*)
(* :Package Version: .2	(7/15/92 Alpha Release)
*)
(* :Copyright: Copyright 1992 by Robert J. Korsan
This package may be copied in its entirety for nonprofit purposes only.
Sale, other than for the direct cost of the media, is prohibited.  This
copyright notice must accompany all copies.
The author and Springer Verlag, make no representations, express or
implied, with respond to this documentation, of the software it
describes and contains, including without limitations, any implied
warranties of mechantability or fitness for a particular purpose, all
of which are expressly disclaimed.  The author and Springer Verlag, their
licensees, distributors and dealers shall in no event be liable for any
indirect, incidental, or consequential damages.
*)
(* :History:
	Version .1 by Robert J. Korsan, July 1992
	Version .7 by Robert J. Korsan, October 1992
*)
(*:Keywords: continuous distribution, Johnson System, moments *)
(*:Requirements: No special system requirements. *)
(*:Warning:
  This package extends the definition of several descriptive statistics
  functions and the definition of Random.  If the original usage messages
  are reloaded, this change will not be reflected in the usage message,
  although the extended functionality will remain.
*)
(*:Note: Most functions provide numerical results. *)
 
(*:Sources:
  Norman L. Johnson and Samuel Kotz, Continuous Univariate
    Distributions, 1970
    
  Robert Korsan: 'Decision Analytica' in "Econometric and Finanace in
	Mathematica", edited by Dr. Hal Varian, Springer Verlag
*)
BeginPackage["DecisionAnalytica`JohnsonDistributions`",
             "Statistics`InverseStatisticalFunctions`"]
Off[General::spell1];
	(*  Distributions  *)
MomentsToJohnsonDistribution::usage =
"MomentsToJohnsonDistribution[m1, m2, m3, m4] assumes that m1, m2, m3 and m4 are
the first four moments (about zero) of some probability distribution. It then proceeds to test
whether the domain of the corresponding Johnson system distribution is bounded 
(finite subset of the real line), semi-bounded (one end infinite) or unbounded 
(-Infinity to +Infinity). Finally, it computes the appropriate numerical parameters
of the Johnson distribution and returns the appropriate Mathematica object."
JohnsonBoundedDistribution::usage =
"JohnsonBoundedDistribution[gamma, delta, xi, lambda] represents the continuous 
Johnson distribution whose domain is a finite interval of the real line with 
parameters gamma, delta, xi and lambda."
JohnsonLogNormalDistribution::usage =
"JohnsonLogNormalDistribution[gamma, delta, xi, 1] represents the Johnson Log-Normal 
distribution whose domain is a half infinite sub-interval of the real line with
parameters gamma, delta, xi and (lambda === 1)."
JohnsonUnboundedDistribution::usage =
"JohnsonUnboundedDistribution[gamma, delta, xi, lambda] represents the continuous 
Johnson distribution whose domain is the real line with parameters gamma, delta, 
xi and lambda."
	(*   Functions  *)
(* Introduce usage messages for new functions. *)
Domain::usage =
"Domain[distribution] gives the domain of the specified distribution."
 
PDF::usage =
"PDF[distribution, x] gives the probability density function of
the specified statistical distribution evaluated at x."
 
CDF::usage =
"CDF[distribution, x] gives cumulative distribution function of distribution
at the point x, defined as the integral of the probability density of the
the distribution from the lowest value in the domain to x."
 
(*
  Extend usage messages for existing functions, if this has
  not already been done.
*)
If[StringPosition[Mean::usage, "specified statistical distribution"] === {},
Mean::usage = Mean::usage <> " " <>
"Mean[distribution] gives the mean of the specified statistical distribution.";
Median::usage = Median::usage <> " " <>
"Median[distribution] gives the Median of the specified statistical distribution.";
Variance::usage = Variance::usage <> " " <>
"Variance[distribution] gives the variance of the specified
statistical distribution.";
StandardDeviation::usage = StandardDeviation::usage <> " " <>
"StandardDeviation[distribution] gives the standard deviation of the
specified statistical distribution.";
Skewness::usage = Skewness::usage <> " " <>
"Skewness[distribution] gives the coefficient of skewness of the
specified statistical distribution.";
Kurtosis::usage = Kurtosis::usage <> " " <>
"Kurtosis[distribution] gives the coefficient of kurtosis of the
specified statistical distribution.";
KurtosisExcess::usage = KurtosisExcess::usage <> " " <>
"KurtosisExcess[distribution] gives the kurtosis excess for the
specified statistical distribution.";
Quantile::usage =
"Quantile[distribution, q] gives the q-th quantile of the specified
statistical distribution."
]
(* Extend the usage message of Random, if this has not already been done. *)
If[StringPosition[Random::usage, "specified statistical distribution"] === {},
Random::usage = Random::usage <> " " <>
"Random[distribution] gives a random number with the specified
statistical distribution."
]
Begin["`Private`"]
(* needed calcs *)
MomentsToJohnsonDistribution[m1_, m2_, m3_, m4_] :=
 Module[{mu1, mu2, mu3, mu4, b1, b2, w, wRules, wval, beta2, delta,
         gamma,  xi, lambda, Om, g, d, muy1, muy2},
   (* find central moments from moments about zero *)
   {mu1, mu2, mu3, mu4} = Centralize[m1, m2, m3, m4];
   
   (* find skewness and kurtosis *)
   b1 = mu3^2/mu2^3; b2 = mu4/mu2^2;
   
   (* dividing line is "LogNormal" distribution                  *)
   (* test whether skewness and kurtosis fall on the 'line'      *)
   (* assume skewness (b1) is on the 'line', find                *)
   (* kurtosis (beta2 on the line) and compare actual            *)
   (* kurtosis (b2). If b2 > beta2 then unbounded.               *)
   (* If b2 == beta2 then LogNormal. If b2 < beta2 then bounded. *)
   
   wRules = NSolve[ b1 == (w-1)(w+2)^2, w];
   
   (* By Descartes' Rule for the number of *)
   (* sign changes there is one real root. *)
   wval  = First[Select[w/.wRules, 0 == Im[Chop[#]]&]];
   beta2 = {1, 2, 3, -3}.(wval^{4, 3, 2, 0});
   
   (* Case 1: LogNormal result. *)
   If[ Abs[beta2 - b2] <= 10^-6, delta = 1/Sqrt[Log[wval]];
       gamma = (2 - delta^2*Log[mu2])/(2*delta);
       xi = m1 - Exp[1/(2*delta^2) - gamma/delta];
       Return[JohnsonLogNormalDistribution[gamma, delta, xi, 1]]];
   
   (* Case 2: b2 < beta2 implies bounded solution *) 
   If[ b2 < beta2,
         {gamma, delta} = {g, d}/.FindRoot[{0==b1-f1[g,d], 0==b2-f2[g,d]},
                    {g,Random[]},{d,Random[]},
                    Jacobian->JohnsonBoundedDistributionHessian[g,d],
                    DampingFactor->1];
         {muy1, muy2} = {JohnsonBoundedDistributionMoment[1, gamma, delta],
                         JohnsonBoundedDistributionMoment[2, gamma, delta]};
         lambda = Sqrt[(m2 -m1^2)/(muy2 - muy1^2)];
         xi = m1 - lambda*muy1;
         Return[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]
   ];
   
   (* Case 2: b2 > beta2 implies unbounded solution *)
   {gamma, delta, wval, Om} = UnboundedGDFromBeta[b1, b2];
   lambda = Sqrt[mu2/((wval-1)(wval*Cosh[2*Om]+1)/2)];
   xi = m1 + Sqrt[wval]*Sinh[Om];
   Return[JohnsonUnboundedDistribution[gamma, delta, xi, lambda]]
 ]
 
 Centralize[m1_, m2_, m3_, m4_] :=
  Module[{mu1, mu2, mu3, mu4},
   mu1 = 0; mu2 = m2 - m1^2; mu3 = m3 - 3 m2 m1 + 2 m1^3;
   mu4 = m4 -4 m3 m1 +6 m2 m1^2 - 3 m1^4;
   {mu1, mu2, mu3, mu4}]
 
 JohnsonBoundedDistributionMoment[r_, g_, d_]:=
   JohnsonBoundedDistributionMoment[r, g, d]=
      NIntegrate[ Exp[-z^2/2]/(1+Exp[-(z-g)/d])^r, {z,-Infinity,Infinity}]/N[Sqrt[2Pi]]
 
JohnsonBoundedDistributionHessian[g_,d_]:=JBDHessian[g,d]
f1[g_,d_]:=
Module[{mu1,mu2,mu3,mu4,m5,mu6,b1,b2,s, 
        dmu3dg,dmu4dg,dmu3dd,dmu4dd,
        db1dg,db1dd,db2dg,db2dd},
 mu1=JohnsonBoundedDistributionMoment[1,g,d];
 mu2=JohnsonBoundedDistributionMoment[2,g,d];
 mu3=JohnsonBoundedDistributionMoment[3,g,d];
 mu4=JohnsonBoundedDistributionMoment[4,g,d];
 mu5=JohnsonBoundedDistributionMoment[5,g,d];
 mu6=JohnsonBoundedDistributionMoment[6,g,d];
 
 s=Sqrt[mu2-mu1^2];
 b1=(mu3 - 3 mu2 mu1 + 2 mu1^3)/s^3;
 b2=(mu4 - 4 mu3 mu1 + 6 mu2 mu1^2 - 3 mu1^4)/s^4;
 
 dmu1dg= (mu2-mu1)/d;
 dmu1dd= ((g*d-1)(mu1-mu2)+2(mu2-mu3))/d^3;
 dmu2dg=2(mu3-mu2)/d;
 dmu2dd=2((g*d-2)(mu2-mu3)+3(mu3-mu4))/d^3;
 dmu3dg=3(mu4-mu3)/d;
 dmu3dd=3((g*d-3)(mu3-mu4)+4(mu4-mu5))/d^3;
 dmu4dg=4(mu5-mu4)/d;
 dmu4dd=4((g*d-4)(mu4-mu5)+5(mu5-mu6))/d^3;
 
 db1dg=(-2*dmu3dg*mu1^2 + 2*dmu3dg*mu2 + 3*dmu2dg*mu1*mu2 - 6*dmu1dg*mu2^2 - 
    3*dmu2dg*mu3 + 6*dmu1dg*mu1*mu3)/(2*(mu1^2 - mu2)^2*(-mu1^2 + mu2)^(1/2));
    
 db1dd=(-2*dmu3dd*mu1^2 + 2*dmu3dd*mu2 + 3*dmu2dd*mu1*mu2 - 6*dmu1dd*mu2^2 - 
    3*dmu2dd*mu3 + 6*dmu1dd*mu1*mu3)/(2*(mu1^2 - mu2)^2*(-mu1^2 + mu2)^(1/2));
 
 db2dg=(dmu4dg - 4*dmu3dg*mu1 + 6*dmu2dg*mu1^2 - 12*dmu1dg*mu1^3 + 12*dmu1dg*mu1*mu2 - 
     4*dmu1dg*mu3)/(mu1^2 - mu2)^2 + 
  (2*(dmu2dg - 2*dmu1dg*mu1)*(-3*mu1^4 + 6*mu1^2*mu2 - 4*mu1*mu3 + mu4))/
   (mu1^2 - mu2)^3;
 
 db2dd=(dmu4dd - 4*dmu3dd*mu1 + 6*dmu2dd*mu1^2 - 12*dmu1dd*mu1^3 + 12*dmu1dd*mu1*mu2 - 
     4*dmu1dd*mu3)/(mu1^2 - mu2)^2 + 
  (2*(dmu2dd - 2*dmu1dd*mu1)*(-3*mu1^4 + 6*mu1^2*mu2 - 4*mu1*mu3 + mu4))/
   (mu1^2 - mu2)^3;
 
 JBDHessian[g,d]= {{db1dg, db1dd}, {db2dg, db2dd}};
     
     (*Print[{g,d,b1,b2}];
     Print[JBDHessian[g,d]];*)
 f2[g,d]=b2; b1
]
UnboundedGDFromBeta[b1_, b2_] :=
 Module[{w0, Om0, w, Om, wOmRules, d, g},
   {w0, Om0} = {1.667255764425944132 - 0.03645973459827422399*b1 + 0.01361515642943933493*b2 + 
                0.03645973459827422399*(79.991094706098823 + 9.227261837260119049*b1 + 1.*b1^2 + 1.433424651430158884*b2 - 
                0.7468598759401716192*b1*b2 + 0.1394499185723421355*b2^2)^(1/2), 
               -1.030074812194869865 + 0.1221188839888251221*b1 - 0.04560284727292307048*b2 + 
                0.1221188839888251221*(79.991094706098823 + 9.227261837260119049*b1 + 1.*b1^2 + 1.433424651430158884*b2 - 
                0.7468598759401716192*b1*b2 + 0.1394499185723421355*b2^2)^(1/2)};
   Off[FindRoot::precw];
   wOmRules=FindRoot[{b1*((w-1)(1+w*Cosh[2 Om])/2)^3 == (-Sqrt[w] (w-1)^2(w(w+2)Sinh[3 Om] + 3 Sinh[Om])/4)^2,
                      b2*((w-1)(1+w*Cosh[2 Om])/2)^2 ==  (w-1)^2(w^2(w^4 + 2w^3 + 3w^2 -3)Cosh[4 Om] + 4 w^2(w+2)Cosh[2 Om] + 3(1+2w))/8},
                      {w, w0}, {Om, Om0}];
   On[FindRoot::precw];
   w0 = w/.wOmRules;
   d = 1/Sqrt[Log[w0]];
   g = d*Om/.wOmRules; If[Abs[g] <= 10^-3, g=0]; {g, d, w0}
 ]
 
 
(* Continuous Johnson Bounded Distribution *)
JohnsonBoundedDistribution/: 
  Domain[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] := {xi,xi+lambda}
JohnsonBoundedDistribution/: PDF[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_], x_] :=
  Module[{result, t, low, high, y},
 	If[NumberQ[N[x]], low = If[Chop[xi] == 0, 10^-12, xi*(1+10^-12)];
  	   high=If[0==Chop[xi+lambda], Sign[xi]*10^-12, (1-10^-12)*(xi+lambda)];
  	   t=Min[high, Max[low, x]],
  	   t=x];
	y = (t-xi)/lambda;
	result=delta*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(lambda*Sqrt[2Pi] y(1-y));
  	If[NumberQ[N[result]], N[result], result]]
JohnsonBoundedDistribution/: CDF[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_], x_] := 
  Module[{result, t, low, high},
 	If[NumberQ[N[x]], low=If[Chop[xi]==0, 10^-12, xi*(1+10^-12)];
  	   high=If[0==Chop[xi+lambda], Sign[xi]*10^-12, (1-10^-12)*(xi+lambda)];
  	   t=Min[high, Max[low, x]],
  	   t=x];
	y = (t-xi)/lambda;
  	result = (1+Erf[(gamma+delta Log[ y/(1-y)])/Sqrt[2]])/2;
 	 If[NumberQ[N[result]], N[result], result]] 
JohnsonBoundedDistribution/: Mean[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] := Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]] = 
Block[{result},
  Off[NIntegrate::precw];
  result = If[ (NumberQ[gamma] && NumberQ[delta] && NumberQ[xi] && NumberQ[lambda]),
       N[xi+lambda*delta*NIntegrate[Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] (1-y)), {y, 10^-12, 1-10^-12}]],
       (* Else *)
       Literal[xi+lambda(1/Sqrt[2 Pi])Exp[-gamma^2/2]* (delta^(-1)/2 + delta^(-1) * 
        Sum[ Exp[-n^2/(2 delta^2)] Cosh[n (1-2 gamma delta)/(2 delta^2)] Sech[n/(2 delta^2)], 
            {n, 1, Infinity}]
         - 2 Pi delta Sum[Exp[-(2n-1)^2 Pi^2 delta^2/2] Sin[(2 n-1) Pi gamma delta] Csch[(2n-1)Pi^2 delta^2], {n, 1, Infinity}])/
         (1 + 2 Sum[Exp[-2 n^2 Pi^2 delta^2] Cos[2 n Pi gamma delta], {n, 1, Infinity}])]/.n->Unique["n"]];
   On[NIntegrate::precw]; result
 ]
JohnsonBoundedDistribution/: Median[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    xi+lambda/(1+Exp[gamma/delta])//N
JohnsonBoundedDistribution/: Variance[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] :=
   Module[{mux2, mux1},
    If[ NumberQ[gamma] && NumberQ[delta] && NumberQ[xi] && NumberQ[lambda],
       mux1 = Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       mux2 = delta*NIntegrate[(xi+lambda*y)^2*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] y(1-y)), {y, 10^-12, 1-10^-12}]//N;
       Variance[JohnsonBoundedDistribution[gamma, delta, xi, lambda]] = mux2-mux1^2,
    (* Else *)
       (2 xi + lambda) HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]
                 -lambda*xi - xi^2 - HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]^2] +
                 lambda*delta*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]],gamma]]]]
JohnsonBoundedDistribution/: StandardDeviation[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    If[ NumberQ[gamma] && NumberQ[delta] && NumberQ[xi] && NumberQ[lambda],
       Sqrt[Variance[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]],
    (* Else *)
       Sqrt[(2 xi + lambda) HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]
                 -lambda*xi - xi^2 - HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]^2] +
                 lambda*delta*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]],gamma]]]]
JohnsonBoundedDistribution/: Skewness[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{mux3, mux2, mux1, s2},
    If[ NumberQ[gamma] && NumberQ[delta] && NumberQ[xi] && NumberQ[lambda],
       mux3 = N[delta*NIntegrate[(xi+lambda*y)^3*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] y(1-y)),
        {y, 10^-12, 1-10^-12}]];
       mux1 = Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       s2 = Variance[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       mux2 = s2 + mux1^2;
       (mux3 - 3 mux2 mux1 + 2 mux1^3)^2/s2^3,
    (* Else *)
       (-(lambda^2*xi) - 3*lambda*xi^2 - 2*xi^3 + (3*delta*lambda^2*
        HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]])/2 + 
     3*delta*lambda*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
     (delta^2*lambda^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]])/2 + 
     lambda^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
     6*lambda*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
     6*xi^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
     3*delta*lambda*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
      HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
     3*lambda*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 - 
     6*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 + 
     2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^3)^2/
  (-(lambda*xi) - xi^2 + delta*lambda*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
     lambda*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
     2*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2)^3]]
JohnsonBoundedDistribution/: Kurtosis[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{mux4, mux3, mux2, mux1, s2},
    If[ NumberQ[gamma] && NumberQ[delta] && NumberQ[xi] && NumberQ[lambda],
       mux4 = N[delta*NIntegrate[(xi+lambda*y)^4*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] y(1-y)),
        {y, 10^-12, 1-10^-12}]];
       mux3 = N[delta*NIntegrate[(xi+lambda*y)^3*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] y(1-y)),
        {y, 10^-12, 1-10^-12}]];
       mux1 = Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       s2 = Variance[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       mux2 = s2 + mux1^2;
       (mux4 - 4 mux3 mux1 + 6 mux2 mux1^2 - 3 mux1^4)/s2^2,
    (* Else *)
       (-(lambda^3*xi) - 4*lambda^2*xi^2 - 6*lambda*xi^3 - 3*xi^4 + 
    (11*delta*lambda^3*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]])/6 + 
    6*delta*lambda^2*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
    6*delta*lambda*xi^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
    delta^2*lambda^3*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]] + 
    2*delta^2*lambda^2*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]] + 
    (delta^3*lambda^3*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 3}]])/6 + 
    lambda^3*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
    8*lambda^2*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
    18*lambda*xi^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
    12*xi^3*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    6*delta*lambda^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    12*delta*lambda*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    2*delta^2*lambda^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    4*lambda^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 - 
    18*lambda*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 - 
    18*xi^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 + 
    6*delta*lambda*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 + 
    6*lambda*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^3 + 
    12*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^3 - 
    3*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^4)/
  (-(lambda*xi) - xi^2 + delta*lambda*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
     lambda*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
     2*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2)^2]]
JohnsonBoundedDistribution/: KurtosisExcess[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{mux4, mux3, mux2, mux1, s2},
    If[ NumberQ[gamma] && NumberQ[delta] && NumberQ[xi] && NumberQ[lambda],
       mux4 = N[delta*NIntegrate[(xi+lambda*y)^4*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] y(1-y)),
        {y, 10^-12, 1-10^-12}]];
       mux3 = N[delta*NIntegrate[(xi+lambda*y)^3*Exp[-(gamma+delta*Log[y/(1-y)])^2/2]/(Sqrt[2Pi] y(1-y)),
        {y, 10^-12, 1-10^-12}]];
       mux1 = Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       s2 = Variance[JohnsonBoundedDistribution[gamma, delta, xi, lambda]];
       mux2 = s2 + mux1^2;
       -3 + (mux4 - 4 mux3 mux1 + 6 mux2 mux1^2 - 3 mux1^4)/s2^2,
    (* Else *)
       -3 + (-(lambda^3*xi) - 4*lambda^2*xi^2 - 6*lambda*xi^3 - 3*xi^4 + 
    (11*delta*lambda^3*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]])/6 + 
    6*delta*lambda^2*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
    6*delta*lambda*xi^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
    delta^2*lambda^3*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]] + 
    2*delta^2*lambda^2*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]] + 
    (delta^3*lambda^3*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 3}]])/6 + 
    lambda^3*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
    8*lambda^2*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
    18*lambda*xi^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
    12*xi^3*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    6*delta*lambda^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    12*delta*lambda*xi*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    2*delta^2*lambda^2*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], {gamma, 2}]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
    4*lambda^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 - 
    18*lambda*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 - 
    18*xi^2*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 + 
    6*delta*lambda*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]]*
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2 + 
    6*lambda*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^3 + 
    12*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^3 - 
    3*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^4)/
  (-(lambda*xi) - xi^2 + delta*lambda*HoldForm[D[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]], gamma]] + 
     lambda*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] + 
     2*xi*HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]] - 
     HoldForm[Mean[JohnsonBoundedDistribution[gamma, delta, xi, lambda]]]^2)^2]]
JohnsonBoundedDistribution/:
        CharacteristicFunction[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_], t_] :=
 	NotImplemented /; False
JohnsonBoundedDistribution/: Quantile[JohnsonBoundedDistribution[gamma_, delta_, xi_, lambda_], q_] :=
    Module[{result, t = Sqrt[2] InverseErf[2 q - 1]},
		result=xi+lambda/(1+Exp[-(t-gamma)/delta]); 
		If[NumberQ[N[result]],N[result],result]]
JohnsonBoundedDistribution/: Random[JohnsonBoundedDistribution[gamma_, delta_, xi_, 1]] :=
    Module[{t = Sqrt[2] InverseErf[2 Random[] - 1]}, xi+lambda/(1+Exp[-(t-gamma)/delta])//N]
(* Johnson Log-Normal Distribution *)
JohnsonLogNormalDistribution/: Domain[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] = {xi, Infinity}
JohnsonLogNormalDistribution/: PDF[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1], x_] :=
 With[{result = delta*Exp[-(gamma+delta*Log[x - xi])^2/2]/((x - xi)*Sqrt[2 Pi]) },
	If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: CDF[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1], x_] :=
    With[{result = (Erf[(gamma + delta*Log[x-xi]) / Sqrt[2]] + 1)/2},
	 If[ NumberQ[N[result]], N[result], result] ]
JohnsonLogNormalDistribution/: Mean[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
	With[{result = gamma + delta*Exp[1/(2*delta^2)-(gamma/delta)]},
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: Variance[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
	With[{muy1 = Exp[1/(2*delta^2)-(gamma/delta)], muy2 = Exp[4/(2*delta^2)-(2*gamma/delta)], result},
		result = lambda^2 (muy2 - muy1^2);
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: StandardDeviation[
                JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
	With[{result = Sqrt[Variance[JohnsonLogNormalDistribution[gamma, delta, xi, 1]]]},
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: Skewness[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
	With[{result, w = Exp[delta^-2]},
		result = (w - 1)(w+2)^2;
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: Kurtosis[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
	With[{result, w = Exp[delta^-2]},
		result = w^4 + 2 w^3 + 3 w^2 -3;
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: KurtosisExcess[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
   With[{result, w = Exp[delta^-2]},
		result = w^4 + 2 w^3 + 3 w^2 -6;
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: CharacteristicFunction[
                JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1], t_] :=
        NotImplemented /; False
JohnsonLogNormalDistribution/: Quantile[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1], q_] :=
	With[{result =  xi + Exp[(InverseErf[2 q - 1] - gamma)/delta]},
		If[NumberQ[N[result]], N[result], result]]
JohnsonLogNormalDistribution/: Random[JohnsonLogNormalDistribution[gamma_, delta_, xi_, 1]] :=
	With[{result = xi + Exp[(InverseErf[2 Random[] - 1]-gamma)/delta]},
		If[NumberQ[N[result]], N[result], result]]
(* Continuous Johnson Unbounded Distribution *)
JohnsonUnboundedDistribution/: 
  Domain[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] := {-Infinity, Inifnity}
JohnsonUnboundedDistribution/: PDF[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_], x_] :=
  Module[{result, t, low, high, y},
 	If[NumberQ[N[x]], low = If[Chop[xi] == 0, 10^-12, xi*(1+10^-12)];
  	   high=If[0==Chop[xi+lambda], Sign[xi]*10^-12, (1-10^-12)*(xi+lambda)];
  	   t=Min[high, Max[low, x]],
  	   t=x];
	y = (t-xi)/lambda;
	result=(delta/(lambda*Sqrt[2 Pi (1+y^2)]))Exp[-(gamma+delta*Log[y+Sqrt[1+y^2]])^2/2];
  	If[NumberQ[N[result]], N[result], result]]
JohnsonUnboundedDistribution/: CDF[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_], x_] := 
  Module[{result, t, low, high},
 	If[NumberQ[N[x]], low=If[Chop[xi]==0, 10^-12, xi*(1+10^-12)];
  	   high=If[0==Chop[xi+lambda], Sign[xi]*10^-12, (1-10^-12)*(xi+lambda)];
  	   t=Min[high, Max[low, x]],
  	   t=x];
	y = (t-xi)/lambda;
  	result = (1+Erf[(gamma+delta*ArcSinh[y])/Sqrt[2]])/2;
 	 If[NumberQ[N[result]], N[result], result]] 
JohnsonUnboundedDistribution/: Mean[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] := Mean[JohnsonUnboundedDistribution[gamma, delta, xi, lambda]] = 
Module[{result, w, Om, muy1},
  w = Exp[delta-2]; Om = gamma/delta;
  muy1 = - Sqrt[w] Sinh[Om]; result = gamma + delta*muy1;
  If[ NumberQ[N[result]], N[result], result]
 ]
JohnsonUnboundedDistribution/: Median[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    xi+lambda*(-Sinh[gamma/delta])//N
JohnsonUnboundedDistribution/: Variance[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
   Module[{result, muy2, w, Om},
     w = Exp[delta-2]; Om = gamma/delta;
     muy2 = (w-1)(1 + w Cosh[2 Om])/2; result = lambda^2 * muy2;
     If[ NumberQ[N[result]], N[result], result]
   ]
JohnsonUnboundedDistribution/: StandardDeviation[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Sqrt[Variance[JohnsonUnboundedDistribution[gamma, delta, xi, lambda]]]
JohnsonUnboundedDistribution/: Skewness[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{result, muy2, muy4, w, Om},
      w = Exp[delta-2]; Om = gamma/delta;
      muy2 = (w-1)(1 + w Cosh[2 Om])/2;
      muy3 = -Sqrt[w] (w-1)^2 (w(w+2) Sinh[3 Om] + 3 Sinh[Om])/4;
      result = muy3^2 / muy2^3;
      If[ NumberQ[N[result]], N[result], result]
    ]
JohnsonUnboundedDistribution/: Kurtosis[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{result, muy2, muy, w, Om},
      w = Exp[delta-2]; Om = gamma/delta;
      muy2 = (w-1)(1 + w Cosh[2 Om])/2;
      muy4 = (w-1)^2 (w^2 (w^4 + 2 w^3 +3 w^2 - 3)Cosh[4 Om] + 4 w^2 Cosh[2 Om] + 3(2w+1))/8;
      result = muy4 / muy2^2;
      If[ NumberQ[N[result]], N[result], result]
    ]
JohnsonUnboundedDistribution/: KurtosisExcess[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{result, muy2, muy, w, Om},
      w = Exp[delta-2]; Om = gamma/delta;
      muy2 = (w-1)(1 + w Cosh[2 Om])/2;
      muy4 = (w-1)^2 (w^2 (w^4 + 2 w^3 +3 w^2 - 3)Cosh[4 Om] + 4 w^2 Cosh[2 Om] + 3(2w+1))/8;
      result = -3 + muy4 / muy2^2;
      If[ NumberQ[N[result]], N[result], result]
    ]
JohnsonUnboundedDistribution/:
        CharacteristicFunction[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_], t_] :=
 	NotImplemented /; False
JohnsonUnboundedDistribution/: Quantile[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_], q_] :=
    Module[{result, t = Sqrt[2] InverseErf[2 q - 1]},
		result=xi+lambda*Sinh[(t-gamma)/delta]; 
		If[NumberQ[N[result]],N[result],result]]
JohnsonUnboundedDistribution/: Random[JohnsonUnboundedDistribution[gamma_, delta_, xi_, lambda_]] :=
    Module[{t = Sqrt[2] InverseErf[2 Random[] - 1]}, xi+lambda*Sinh[(t-gamma)/delta]//N]

End[]
(*SetAttributes[ JohnsonBoundedDistribution, ReadProtected];

Protect[JohnsonBoundedDistribution,
	JohnsonLogNormalDistribution,];*)
On[General::spell1];
EndPackage[]
